@@ -18,32 +18,37 @@ from flare.env import AtomicEnvironment
 from flare.gp import GaussianProcess
 from flare.mgp.mgp import MappedGaussianProcess
 from flare.output import Output
-from flare.predict import predict_on_atom, predict_on_atom_en, \
-    predict_on_structure_par, predict_on_structure_par_en, \
-    predict_on_structure_mgp
+from flare.predict import (
+    predict_on_atom,
+    predict_on_atom_en,
+    predict_on_structure_par,
+    predict_on_structure_par_en,
+    predict_on_structure_mgp,
+)
 from flare.struc import Structure
 from flare.utils.element_coder import element_to_Z, Z_to_element, NumpyEncoder
-from flare.utils.learner import subset_of_frame_by_element, \
-    is_std_in_bound_per_species, is_force_in_bound_per_species
+from flare.utils.learner import (
+    subset_of_frame_by_element,
+    is_std_in_bound_per_species,
+    is_force_in_bound_per_species,
+)
 from flare.mgp import MappedGaussianProcess
 from flare.parameters import Parameters
 from flare.output import Output
 
+
 class LearningProtocol(object):
-
-    def __init__(self, model: Union[GaussianProcess,MappedGaussianProcess],
-                    n_cpus : int = 0,
-                 protocol_name: str = 'learning',
-                 )
-
-
+    def __init__(
+        self,
+        model: Union[GaussianProcess, MappedGaussianProcess],
+        n_cpus: int = 0,
+        protocol_name: str = "learning",
+    ):
 
         self.model = model
-        self.output = Output(basename=protocol_name,
-                             always_flush=True)
+        self.output = Output(basename=protocol_name, always_flush=True)
 
-
-    def train_model(self,**train_kwargs):
+    def train_model(self, **train_kwargs):
 
         if isinstance(self.model, GaussianProcess):
             self.model.train(train_kwargs)
@@ -53,23 +58,27 @@ class LearningProtocol(object):
 
         raise NotImplementedError("Must be implemented in child class")
 
-    def augment_training_set(self, structure: Structure,
-                             selective_atoms: List [int] = list(),
-                             forces: Iterable[float] = None,
-                             energy: float = None,
-                             ):
+    def augment_training_set(
+        self,
+        structure: Structure,
+        selective_atoms: List[int] = list(),
+        forces: Iterable[float] = None,
+        energy: float = None,
+    ):
 
         if forces is None:
             forces = structure.forces
         if energy is None and forces is None:
             energy = structure.energy
 
-        if isinstance(self.model,GaussianProcess):
+        if isinstance(self.model, GaussianProcess):
 
-            self.model.update_db(structure=structure,
-                                 selective_atoms=selective_atoms,
-                                 forces = forces,
-                                 energy = energy)
+            self.model.update_db(
+                structure=structure,
+                selective_atoms=selective_atoms,
+                forces=forces,
+                energy=energy,
+            )
 
         raise NotImplementedError("Must be implemented in child class")
 
@@ -90,30 +99,30 @@ class LearningProtocol(object):
     def pre_run(self):
 
         if isinstance(self.model, GaussianProcess):
-            self.output.write_header(str(self.gp),
-                                     dt=0,
-                                     Nsteps=len(self.frames),
-                                     structure=None,
-                                     optional={
-                                         'GP Statistics':
-                                             json.dumps(
-                                                 self.gp.training_statistics),
-                                         'GP Name': self.gp.name,
-                                         'GP Write Name':
-                             self.output_name + "_model." + self.model_format})
+            self.output.write_header(
+                str(self.gp),
+                dt=0,
+                Nsteps=len(self.frames),
+                structure=None,
+                optional={
+                    "GP Statistics": json.dumps(self.gp.training_statistics),
+                    "GP Name": self.gp.name,
+                    "GP Write Name": self.output_name + "_model." + self.model_format,
+                },
+            )
 
-    def run_passive_learning(self,
-                             frames: List[Structure] = list(),
-                             envs: List [AtomicEnvironment] = list(),
-                             max_atoms_from_frame: int = inf,
-                             shuffle_frames: bool = False,
-                             max_atoms_added: int = 1000,
-                             max_from_frame_by_species: Dict[str,int]= dict()
-                             ):
+    def run_passive_learning(
+        self,
+        frames: List[Structure] = list(),
+        envs: List[AtomicEnvironment] = list(),
+        max_atoms_from_frame: int = inf,
+        shuffle_frames: bool = False,
+        max_atoms_added: int = 1000,
+        max_from_frame_by_species: Dict[str, int] = dict(),
+    ):
 
         if not isinstance(self.model, GaussianProcess):
-            raise NotImplementedError("Passive learning not"
-                                      "yet configured for MGP")
+            raise NotImplementedError("Passive learning notyet configured for MGP")
 
         self.start_time = time.time()
         logger = logging.getLogger(self.logger_name)
@@ -126,7 +135,7 @@ class LearningProtocol(object):
             logger.info("Added {str(env)} to dataset.")
 
         if shuffle_frames:
-            frames = random.sample(frames,len(frames))
+            frames = random.sample(frames, len(frames))
 
         n_atoms_added = 0
 
@@ -139,94 +148,89 @@ class LearningProtocol(object):
                 atoms_of_specie = frame.indices_of_specie(species_i)
                 n_spec = len(atoms_of_specie)
                 # Determine how many to add based on user defined cutoffs
-                n_to_add = min(n_spec,
-                               max_from_frame_by_species.get(species_i, inf),
-                               max_atoms_from_frame)
+                n_to_add = min(
+                    n_spec,
+                    max_from_frame_by_species.get(species_i, inf),
+                    max_atoms_from_frame,
+                )
 
                 train_atoms += random.sample(atoms_of_specie, n_spec)
                 n_atoms_added += n_spec
 
-            self.model.update_db(struc=frame,
-                                 custom_range=train_atoms
-                                 )
+            self.model.update_db(struc=frame, custom_range=train_atoms)
             if n_atoms_added >= max_atoms_added:
                 break
 
         if n_atoms_added:
-            logger.info(f"Added {n_atoms_added} atoms to "
-                        f"pretrain.\n"
-                        f"Pre-run GP Statistics: "
-                        f"{json.dumps(self.model.training_statistics)} ")
+            logger.info(
+                f"Added {n_atoms_added} atoms to "
+                "pretrain.\n"
+                "Pre-run GP Statistics: "
+                f"{json.dumps(self.model.training_statistics)} "
+            )
 
-        if (envs or n_atoms_added or frames) and \
-                (self.pre_train_max_iter or self.max_trains):
-            logger.debug("Now commencing pre-run training of GP (which has "
-                         "non-empty training set)")
+        if (envs or n_atoms_added or frames) and (
+            self.pre_train_max_iter or self.max_trains
+        ):
+            logger.debug(
+                "Now commencing pre-run training of GP (which has "
+                "non-empty training set)"
+            )
             time0 = time.time()
             self.train_gp(max_iter=self.pre_train_max_iter)
             logger.debug(f"Done train_gp {time.time()-time0}")
         else:
-            logger.debug("Now commencing pre-run set up of GP (which has "
-                         "non-empty training set)")
+            logger.debug(
+                "Now commencing pre-run set up of GP (which has non-empty training set)"
+            )
             time0 = time.time()
             self.gp.check_L_alpha()
             logger.debug(f"Done check_L_alpha {time.time()-time0}")
 
         if self.model_format and not self.mgp:
-            self.gp.write_model(f'{self.output_name}_prerun',
-                                self.model_format)
+            self.gp.write_model(f"{self.output_name}_prerun", self.model_format)
 
 
 class TrainFromAIMD(LearningProtocol):
-
     def __init__(self):
-        self
+        pass
 
     def train_model(self):
-
-
-
+        pass
 
 
 class LearningProtocol:
-
-    def __init__(self, gp: Union[GaussianProcess, MappedGaussianProcess],
-                 active_frames: List[Structure] = None,
-                 passive_frames: List[Structure] = None,
-                 passive_envs: List[Tuple[AtomicEnvironment,
-                                          'np.array']] = None,
-
-                 active_rel_var_tol: float = 4,
-                 active_abs_var_tol: float = 1,
-                 active_abs_error_tol: float = 0,
-                 active_error_tol_cutoff: float = inf,
-                 active_max_trains: int = np.inf,
-                 active_max_element_from_frame: dict = None,
-
-                 checkpoint_interval_train: int = 1,
-                 checkpoint_interval_atom: int = 100,
-
-                 predict_atoms_per_element: dict = None,
-
-                 max_atoms_from_frame: int = np.inf,
-                 min_atoms_added_per_train: int = 1,
-                 max_model_size: int = np.inf,
-
-                 passive_on_active_skips: int = -1,
-                 passive_train_max_iter: int = 50,
-                 passive_atoms_per_element: dict = None,
-
-                 active_skip: int = 1,
-                 shuffle_active_frames: bool = False,
-
-                 n_cpus: int = 1,
-                 validate_ratio: float = 0.0,
-                 calculate_energy: bool = False,
-
-                 output_name: str = 'gp_from_aimd',
-                 print_as_xyz: bool = False,
-                 verbose: str = "INFO",
-                 written_model_format: str = 'json'):
+    def __init__(
+        self,
+        gp: Union[GaussianProcess, MappedGaussianProcess],
+        active_frames: List[Structure] = None,
+        passive_frames: List[Structure] = None,
+        passive_envs: List[Tuple[AtomicEnvironment, "np.array"]] = None,
+        active_rel_var_tol: float = 4,
+        active_abs_var_tol: float = 1,
+        active_abs_error_tol: float = 0,
+        active_error_tol_cutoff: float = inf,
+        active_max_trains: int = np.inf,
+        active_max_element_from_frame: dict = None,
+        checkpoint_interval_train: int = 1,
+        checkpoint_interval_atom: int = 100,
+        predict_atoms_per_element: dict = None,
+        max_atoms_from_frame: int = np.inf,
+        min_atoms_added_per_train: int = 1,
+        max_model_size: int = np.inf,
+        passive_on_active_skips: int = -1,
+        passive_train_max_iter: int = 50,
+        passive_atoms_per_element: dict = None,
+        active_skip: int = 1,
+        shuffle_active_frames: bool = False,
+        n_cpus: int = 1,
+        validate_ratio: float = 0.0,
+        calculate_energy: bool = False,
+        output_name: str = "gp_from_aimd",
+        print_as_xyz: bool = False,
+        verbose: str = "INFO",
+        written_model_format: str = "json",
+    ):
         """
         Class which trains a GP off of an AIMD trajectory, and generates
         error statistics between the DFT and GP calls.
@@ -316,9 +320,10 @@ class LearningProtocol:
         self.n_cpus = n_cpus
 
         # Output parameters
-        self.output = Output(output_name, verbose,
-                             print_as_xyz=print_as_xyz, always_flush=True)
-        self.logger_name = self.output.basename+'log'
+        self.output = Output(
+            output_name, verbose, print_as_xyz=print_as_xyz, always_flush=True
+        )
+        self.logger_name = self.output.basename + "log"
         self.train_checkpoint_interval = checkpoint_interval_train
         self.atom_checkpoint_interval = checkpoint_interval_atom
 
@@ -336,31 +341,34 @@ class LearningProtocol:
 
         # Parameters for negotiating with the training active_frames
         self.skip = active_skip
-        assert (isinstance(active_skip, int) and active_skip >= 1), \
-            "Skip needs to be a  positive integer."
+        assert (
+            isinstance(active_skip, int) and active_skip >= 1
+        ), "Skip needs to be a  positive integer."
         self.validate_ratio = validate_ratio
-        assert (0 <= validate_ratio <= 1), \
-            "validate_ratio needs to be [0,1]"
+        assert 0 <= validate_ratio <= 1, "validate_ratio needs to be [0,1]"
 
         # Set up for pretraining
         self.pre_train_max_iter = passive_train_max_iter
         self.pre_train_on_skips = passive_on_active_skips
-        self.seed_envs = [] if passive_envs is None else \
-            passive_envs
-        self.seed_frames = [] if passive_frames is None \
-            else passive_frames
+        self.seed_envs = [] if passive_envs is None else passive_envs
+        self.seed_frames = [] if passive_frames is None else passive_frames
 
-        self.pre_train_env_per_species = {} if passive_atoms_per_element \
-                                        is None else passive_atoms_per_element
-        self.train_env_per_species = {} if active_max_element_from_frame \
-                                    is None else active_max_element_from_frame
+        self.pre_train_env_per_species = (
+            {} if passive_atoms_per_element is None else passive_atoms_per_element
+        )
+        self.train_env_per_species = (
+            {}
+            if active_max_element_from_frame is None
+            else active_max_element_from_frame
+        )
 
         # Convert to Coded Species
         if self.pre_train_env_per_species:
             pre_train_species = list(self.pre_train_env_per_species.keys())
             for key in pre_train_species:
-                self.pre_train_env_per_species[element_to_Z(key)] = \
-                    self.pre_train_env_per_species[key]
+                self.pre_train_env_per_species[
+                    element_to_Z(key)
+                ] = self.pre_train_env_per_species[key]
 
         # Defining variables to be used later
         self.curr_step = 0
@@ -418,21 +426,19 @@ class LearningProtocol:
         """
 
         if self.mgp:
-            raise NotImplementedError("Pre-running not"
-                                      "yet configured for MGP")
-        self.output.write_header(str(self.gp),
-                                 dt=0,
-                                 Nsteps=len(self.frames),
-                                 structure=None,
-                                 std_tolerance=(self.rel_std_tolerance,
-                                                self.abs_std_tolerance),
-                                 optional={
-                                     'GP Statistics':
-                                         json.dumps(
-                                             self.gp.training_statistics),
-                                     'GP Name': self.gp.name,
-                                     'GP Write Name':
-            self.output_name + "_model." + self.model_format})
+            raise NotImplementedError("Pre-running notyet configured for MGP")
+        self.output.write_header(
+            str(self.gp),
+            dt=0,
+            Nsteps=len(self.frames),
+            structure=None,
+            std_tolerance=(self.rel_std_tolerance, self.abs_std_tolerance),
+            optional={
+                "GP Statistics": json.dumps(self.gp.training_statistics),
+                "GP Name": self.gp.name,
+                "GP Write Name": self.output_name + "_model." + self.model_format,
+            },
+        )
 
         self.start_time = time.time()
         logger = logging.getLogger(self.logger_name)
@@ -466,42 +472,50 @@ class LearningProtocol:
                 np.random.shuffle(atoms_of_specie)
                 n_at = len(atoms_of_specie)
                 # Determine how many to add based on user defined cutoffs
-                n_to_add = min(n_at, self.pre_train_env_per_species.get(
-                    species_i, inf), self.max_atoms_from_frame)
+                n_to_add = min(
+                    n_at,
+                    self.pre_train_env_per_species.get(species_i, inf),
+                    self.max_atoms_from_frame,
+                )
 
                 for atom in atoms_of_specie[:n_to_add]:
                     train_atoms.append(atom)
                     self.passive_atom_count += 1
 
-            self.update_gp_and_print(frame=frame,
-                                     train_atoms=train_atoms,
-                                     uncertainties=[], train=False)
+            self.update_gp_and_print(
+                frame=frame, train_atoms=train_atoms, uncertainties=[], train=False
+            )
             frame = self.get_next_passive_frame()
 
         logger = logging.getLogger(self.logger_name)
         if self.passive_atom_count > 0:
-            logger.info(f"Added {self.passive_atom_count} atoms to "
-                        f"pretrain.\n"
-                        f"Pre-run GP Statistics: "
-                        f"{json.dumps(self.gp.training_statistics)} ")
+            logger.info(
+                f"Added {self.passive_atom_count} atoms to "
+                "pretrain.\n"
+                "Pre-run GP Statistics: "
+                f"{json.dumps(self.gp.training_statistics)} "
+            )
 
-        if (self.seed_envs or self.passive_atom_count or self.seed_frames) and \
-                (self.pre_train_max_iter or self.max_trains):
-            logger.debug("Now commencing pre-run training of GP (which has "
-                         "non-empty training set)")
+        if (self.seed_envs or self.passive_atom_count or self.seed_frames) and (
+            self.pre_train_max_iter or self.max_trains
+        ):
+            logger.debug(
+                "Now commencing pre-run training of GP (which has "
+                "non-empty training set)"
+            )
             time0 = time.time()
             self.train_gp(max_iter=self.pre_train_max_iter)
             logger.debug(f"Done train_gp {time.time()-time0}")
         else:
-            logger.debug("Now commencing pre-run set up of GP (which has "
-                         "non-empty training set)")
+            logger.debug(
+                "Now commencing pre-run set up of GP (which has non-empty training set)"
+            )
             time0 = time.time()
             self.gp.check_L_alpha()
             logger.debug(f"Done check_L_alpha {time.time()-time0}")
 
         if self.model_format and not self.mgp:
-            self.gp.write_model(f'{self.output_name}_prerun',
-                                self.model_format)
+            self.gp.write_model(f"{self.output_name}_prerun", self.model_format)
 
     def active_run(self):
         """
@@ -517,10 +531,12 @@ class LearningProtocol:
         logger.debug("Commencing run with pre-run...")
         if not self.mgp:
             if len(self.gp) == 0:
-                logger.warning("You are attempting to train a model with no "
-                              "data in your Gausian Process; it is "
-                               "recommended that you begin with "
-                               "a passive training process.")
+                logger.warning(
+                    "You are attempting to train a model with no "
+                    "data in your Gausian Process; it is "
+                    "recommended that you begin with "
+                    "a passive training process."
+                )
 
         self.preparation_for_active_run()
 
@@ -529,7 +545,6 @@ class LearningProtocol:
         cur_atoms_added_write = 0  # Track atoms added for writing
         cur_trains_done_write = 0  # Track training done for writing
 
-
         self.curr_active_frame_index = -1
         cur_frame = self.get_next_active_frame()
         while cur_frame is not None:
@@ -540,7 +555,8 @@ class LearningProtocol:
             # If no predict_atoms_per_element was specified, predict_atoms
             # will be equal to every atom in the frame.
             predict_atoms = subset_of_frame_by_element(
-                cur_frame, self.predict_atoms_per_element)
+                cur_frame, self.predict_atoms_per_element
+            )
 
             # Atoms which are skipped will have NaN as their force / std values
             local_energies = None
@@ -549,19 +565,31 @@ class LearningProtocol:
             # or GP without
             if self.mgp:
                 pred_forces, pred_stds, local_energies = self.pred_func(
-                    structure=cur_frame, mgp=self.gp, write_to_structure=False,
-                    selective_atoms=predict_atoms, skipped_atom_value=np.nan,
-                    energy=True)
+                    structure=cur_frame,
+                    mgp=self.gp,
+                    write_to_structure=False,
+                    selective_atoms=predict_atoms,
+                    skipped_atom_value=np.nan,
+                    energy=True,
+                )
             elif self.calculate_energy:
                 pred_forces, pred_stds, local_energies = self.pred_func(
-                    structure=cur_frame, gp=self.gp, n_cpus=self.n_cpus,
-                    write_to_structure=False, selective_atoms=predict_atoms,
-                    skipped_atom_value=np.nan)
+                    structure=cur_frame,
+                    gp=self.gp,
+                    n_cpus=self.n_cpus,
+                    write_to_structure=False,
+                    selective_atoms=predict_atoms,
+                    skipped_atom_value=np.nan,
+                )
             else:
                 pred_forces, pred_stds = self.pred_func(
-                    structure=cur_frame, gp=self.gp, n_cpus=self.n_cpus,
-                    write_to_structure=False, selective_atoms=predict_atoms,
-                    skipped_atom_value=np.nan)
+                    structure=cur_frame,
+                    gp=self.gp,
+                    n_cpus=self.n_cpus,
+                    write_to_structure=False,
+                    selective_atoms=predict_atoms,
+                    skipped_atom_value=np.nan,
+                )
 
             # Get Error
             dft_forces = cur_frame.forces
@@ -575,12 +603,18 @@ class LearningProtocol:
 
             self.output.write_gp_dft_comparison(
                 curr_step=self.curr_active_frame_index,
-                frame=dummy_frame, start_time=time.time(),
-                dft_forces=dft_forces, dft_energy=dft_energy, error=error,
-                local_energies=local_energies, KE=0)
+                frame=dummy_frame,
+                start_time=time.time(),
+                dft_forces=dft_forces,
+                dft_energy=dft_energy,
+                error=error,
+                local_energies=local_energies,
+                KE=0,
+            )
 
             logger.debug(
-                f'Single frame calculation time {time.time()-frame_start_time}')
+                f"Single frame calculation time {time.time()-frame_start_time}"
+            )
 
             if self.decide_to_update_db():
 
@@ -589,39 +623,45 @@ class LearningProtocol:
                     noise = 0
                 else:
                     noise = Parameters.get_noise(
-                        self.gp.hyps_mask, self.gp.hyps, constraint=False)
+                        self.gp.hyps_mask, self.gp.hyps, constraint=False
+                    )
 
                 std_in_bound, std_train_atoms = is_std_in_bound_per_species(
                     rel_std_tolerance=self.rel_std_tolerance,
                     abs_std_tolerance=self.abs_std_tolerance,
-                    noise=noise, structure=dummy_frame,
+                    noise=noise,
+                    structure=dummy_frame,
                     max_atoms_added=self.max_atoms_from_frame,
-                    max_by_species=self.train_env_per_species)
+                    max_by_species=self.train_env_per_species,
+                )
 
                 # Get max force error atoms
-                force_in_bound, force_train_atoms = \
-                    is_force_in_bound_per_species(
-                        abs_force_tolerance=self.abs_force_tolerance,
-                        predicted_forces=pred_forces,
-                        label_forces=dft_forces,
-                        structure=dummy_frame,
-                        max_atoms_added=self.max_atoms_from_frame,
-                        max_by_species=self.train_env_per_species,
-                        max_force_error=self.max_force_error)
+                force_in_bound, force_train_atoms = is_force_in_bound_per_species(
+                    abs_force_tolerance=self.abs_force_tolerance,
+                    predicted_forces=pred_forces,
+                    label_forces=dft_forces,
+                    structure=dummy_frame,
+                    max_atoms_added=self.max_atoms_from_frame,
+                    max_by_species=self.train_env_per_species,
+                    max_force_error=self.max_force_error,
+                )
 
                 if not std_in_bound or not force_in_bound:
 
                     # -1 is returned from the is_in_bound methods,
                     # so filter that out and the use sets to remove repeats
-                    train_atoms = list(set(std_train_atoms).union(
-                        force_train_atoms) - {-1})
+                    train_atoms = list(
+                        set(std_train_atoms).union(force_train_atoms) - {-1}
+                    )
 
                     # Compute mae and write to output;
                     # Add max uncertainty atoms to training set
                     self.update_gp_and_print(
-                        cur_frame, train_atoms=train_atoms,
+                        cur_frame,
+                        train_atoms=train_atoms,
                         uncertainties=pred_stds[train_atoms],
-                        train=False)
+                        train=False,
+                    )
                     self.cur_atoms_added_train += len(train_atoms)
                     cur_atoms_added_write += len(train_atoms)
                     # Re-train if number of sampled atoms is high enough
@@ -638,23 +678,26 @@ class LearningProtocol:
                     # iteration
                     will_write = False
 
-                    if self.train_checkpoint_interval and \
-                            cur_trains_done_write and \
-                            self.train_checkpoint_interval \
-                        <= cur_trains_done_write:
+                    if (
+                        self.train_checkpoint_interval
+                        and cur_trains_done_write
+                        and self.train_checkpoint_interval <= cur_trains_done_write
+                    ):
                         will_write = True
                         cur_trains_done_write = 0
 
-                    if self.atom_checkpoint_interval \
-                            and cur_atoms_added_write \
-                            and self.atom_checkpoint_interval \
-                        <= cur_atoms_added_write:
+                    if (
+                        self.atom_checkpoint_interval
+                        and cur_atoms_added_write
+                        and self.atom_checkpoint_interval <= cur_atoms_added_write
+                    ):
                         will_write = True
                         cur_atoms_added_write = 0
 
                     if self.model_format and will_write:
-                        self.gp.write_model(f'{self.output_name}_checkpt',
-                                            self.model_format)
+                        self.gp.write_model(
+                            f"{self.output_name}_checkpt", self.model_format
+                        )
 
                 if self.decide_to_checkLalpha():
                     self.gp.check_L_alpha()
@@ -664,12 +707,15 @@ class LearningProtocol:
         self.output.conclude_run()
 
         if self.model_format and not self.mgp:
-            self.gp.write_model(f'{self.output_name}_model',
-                                self.model_format)
+            self.gp.write_model(f"{self.output_name}_model", self.model_format)
 
-    def update_gp_and_print(self, frame: Structure, train_atoms: List[int],
-                            uncertainties: List[int] = None,
-                            train: bool = True):
+    def update_gp_and_print(
+        self,
+        frame: Structure,
+        train_atoms: List[int],
+        uncertainties: List[int] = None,
+        train: bool = True,
+    ):
         """
         Update the internal GP model training set with a list of training
         atoms indexing atoms within the frame. If train is True, re-train
@@ -682,24 +728,24 @@ class LearningProtocol:
         """
 
         # Group added atoms by species for easier output
-        added_species = [Z_to_element(frame.coded_species[at]) for at in
-                         train_atoms]
+        added_species = [Z_to_element(frame.coded_species[at]) for at in train_atoms]
         added_atoms = {spec: [] for spec in set(added_species)}
 
         for atom, spec in zip(train_atoms, added_species):
             added_atoms[spec].append(atom)
 
         logger = logging.getLogger(self.logger_name)
-        logger.info('Adding atom(s) '
-                    f'{json.dumps(added_atoms,cls=NumpyEncoder)}'
-                    ' to the training set.')
+        logger.info(
+            "Adding atom(s) "
+            f"{json.dumps(added_atoms,cls=NumpyEncoder)}"
+            " to the training set."
+        )
 
         if uncertainties is None or len(uncertainties) != 0:
             uncertainties = frame.stds[train_atoms]
 
         if len(uncertainties) != 0:
-            logger.info(f'Uncertainties: '
-                        f'{uncertainties}.')
+            logger.info(f"Uncertainties: {uncertainties}.")
 
         # update gp model; handling differently if it's an MGP
         if not self.mgp:
@@ -720,9 +766,9 @@ class LearningProtocol:
         """
 
         logger = logging.getLogger(self.logger_name)
-        logger.debug('Train GP')
+        logger.debug("Train GP")
 
-        logger_train = self.output.basename+'hyps'
+        logger_train = self.output.basename + "hyps"
 
         # TODO: Improve flexibility in GP training to make this next step
         # unnecessary, so maxiter can be passed as an argument
@@ -738,20 +784,25 @@ class LearningProtocol:
         else:
             self.gp.train(logger_name=logger_train)
 
-        hyps, labels = Parameters.get_hyps(self.gp.hyps_mask, self.gp.hyps, constraint=False,
-                        label=True)
+        hyps, labels = Parameters.get_hyps(
+            self.gp.hyps_mask, self.gp.hyps, constraint=False, label=True
+        )
         if labels is None:
             labels = self.gp.hyp_labels
-        self.output.write_hyps(labels, hyps,
-                               self.start_time,
-                               self.gp.likelihood,
-                               self.gp.likelihood_gradient,
-                               hyps_mask=self.gp.hyps_mask)
+        self.output.write_hyps(
+            labels,
+            hyps,
+            self.start_time,
+            self.gp.likelihood,
+            self.gp.likelihood_gradient,
+            hyps_mask=self.gp.hyps_mask,
+        )
         self.train_count += 1
 
 
-def parse_trajectory_trainer_output(file: str, return_gp_data: bool = False) \
-        -> Union[List[dict], Tuple[List[dict], dict]]:
+def parse_trajectory_trainer_output(
+    file: str, return_gp_data: bool = False
+) -> Union[List[dict], Tuple[List[dict], dict]]:
     """
     Reads output of a TrajectoryTrainer run by frame. return_gp_data returns
     data about GP model growth useful for visualizing progress of model
@@ -763,13 +814,12 @@ def parse_trajectory_trainer_output(file: str, return_gp_data: bool = False) \
         'maes_by_species', optionally, gp_data dictionary
     """
 
-    with open(file, 'r') as f:
+    with open(file, "r") as f:
         lines = f.readlines()
         num_lines = len(lines)
 
     # Get indexes where frames begin, and include the index of the final line
-    frame_indexes = [i for i in range(num_lines) if '-Frame:' in
-                     lines[i]] + [num_lines]
+    frame_indexes = [i for i in range(num_lines) if "-Frame:" in lines[i]] + [num_lines]
 
     frames = []
 
@@ -795,39 +845,42 @@ def parse_trajectory_trainer_output(file: str, return_gp_data: bool = False) \
 
                 frame_atoms.append(split[0])
 
-                frame_positions.append([float(split[1]), float(split[2]),
-                                        float(split[3])])
-                gp_forces.append([float(split[4]), float(split[5]),
-                                  float(split[6])])
-                stds.append(
-                    [float(split[7]), float(split[8]), float(split[9])])
+                frame_positions.append(
+                    [float(split[1]), float(split[2]), float(split[3])]
+                )
+                gp_forces.append([float(split[4]), float(split[5]), float(split[6])])
+                stds.append([float(split[7]), float(split[8]), float(split[9])])
 
-                dft_forces.append([float(split[10]), float(split[11]),
-                                   float(split[12])])
+                dft_forces.append(
+                    [float(split[10]), float(split[11]), float(split[12])]
+                )
 
             # Terminate at blank line between results
             else:
                 break
         # Loop through information in frame after Data
-        for i in range(frame_indexes[n] + len(frame_positions) + 2,
-                       frame_indexes[n + 1]):
+        for i in range(
+            frame_indexes[n] + len(frame_positions) + 2, frame_indexes[n + 1]
+        ):
 
-            if 'Adding atom(s)' in lines[i]:
+            if "Adding atom(s)" in lines[i]:
                 # Splitting to target the 'added atoms' substring
                 split_line = lines[i][15:-21]
                 added_atoms = json.loads(split_line.strip())
 
-            if 'type ' in lines[i]:
+            if "type " in lines[i]:
                 cur_line = lines[i].split()
                 frame_species_maes[cur_line[1]] = float(cur_line[3])
 
-        cur_frame_stats = {'species': frame_atoms,
-                           'positions': frame_positions,
-                           'gp_forces': gp_forces,
-                           'dft_forces': dft_forces,
-                           'gp_stds': stds,
-                           'added_atoms': added_atoms,
-                           'maes_by_species': frame_species_maes}
+        cur_frame_stats = {
+            "species": frame_atoms,
+            "positions": frame_positions,
+            "gp_forces": gp_forces,
+            "dft_forces": dft_forces,
+            "gp_stds": stds,
+            "added_atoms": added_atoms,
+            "maes_by_species": frame_species_maes,
+        }
 
         frames.append(cur_frame_stats)
 
@@ -837,29 +890,29 @@ def parse_trajectory_trainer_output(file: str, return_gp_data: bool = False) \
     # Compute information about GP training
     # to study GP growth and performance over trajectory
 
-    gp_stats_line = [line for line in lines[:30] if 'GP Statistics' in
-                     line and 'Pre-run' not in line][0][15:].strip()
+    gp_stats_line = [
+        line for line in lines[:30] if "GP Statistics" in line and "Pre-run" not in line
+    ][0][15:].strip()
 
     initial_gp_statistics = json.loads(gp_stats_line)
 
     # Get pre_run statistics (if pre-run was done):
     pre_run_gp_statistics = None
-    pre_run_gp_stats_line = [line for line in lines if 'Pre-run GP' in line]
+    pre_run_gp_stats_line = [line for line in lines if "Pre-run GP" in line]
     if pre_run_gp_stats_line:
-        pre_run_gp_statistics = json.loads(pre_run_gp_stats_line[0][
-                                           22:].strip())
+        pre_run_gp_statistics = json.loads(pre_run_gp_stats_line[0][22:].strip())
 
     # Compute cumulative GP size
-    cumulative_gp_size = [int(initial_gp_statistics['N'])]
+    cumulative_gp_size = [int(initial_gp_statistics["N"])]
 
     if pre_run_gp_stats_line:
-        cumulative_gp_size.append(int(pre_run_gp_statistics['N']))
+        cumulative_gp_size.append(int(pre_run_gp_statistics["N"]))
 
     running_total = cumulative_gp_size[-1]
 
     for frame in frames:
 
-        added_atom_dict = frame['added_atoms']
+        added_atom_dict = frame["added_atoms"]
         for val in added_atom_dict.values():
             running_total += len(val)
         cumulative_gp_size.append(running_total)
@@ -867,19 +920,21 @@ def parse_trajectory_trainer_output(file: str, return_gp_data: bool = False) \
     # Compute MAEs for each element over time
     all_species = set()
     for frame in frames:
-        all_species = all_species.union(set(frame['species']))
+        all_species = all_species.union(set(frame["species"]))
 
     all_species = list(all_species)
     mae_by_elt = {elt: [] for elt in all_species}
 
     for frame in frames:
         for elt in all_species:
-            cur_mae = frame['maes_by_species'].get(elt, np.nan)
+            cur_mae = frame["maes_by_species"].get(elt, np.nan)
             mae_by_elt[elt].append(cur_mae)
 
-    gp_data = {'init_stats': initial_gp_statistics,
-               'pre_train_stats': pre_run_gp_statistics,
-               'cumulative_gp_size': cumulative_gp_size,
-               'mae_by_elt': mae_by_elt}
+    gp_data = {
+        "init_stats": initial_gp_statistics,
+        "pre_train_stats": pre_run_gp_statistics,
+        "cumulative_gp_size": cumulative_gp_size,
+        "mae_by_elt": mae_by_elt,
+    }
 
-    return frames, gp_data
\ No newline at end of file
+    return frames, gp_data
