@@ -159,13 +159,17 @@ class MapXbody:
         out_dict.pop("kernel_info")
 
         # only save the mean coefficients and var if var_map == 'simple'
-        if self.var_map == 'simple':
-            out_dict["maps"] = [[m.mean.__coeffs__ for m in self.maps],
-                                [m.var.__coeffs__ for m in self.maps]]
+        if self.var_map == "simple":
+            out_dict["maps"] = [
+                [m.mean.__coeffs__ for m in self.maps],
+                [m.var.__coeffs__ for m in self.maps],
+            ]
         else:
             out_dict["maps"] = [[m.mean.__coeffs__ for m in self.maps]]
-            if self.var_map == 'pca':
-                warnings.warn("var_map='pca' is too heavy to dump, change to var_map=None")
+            if self.var_map == "pca":
+                warnings.warn(
+                    "var_map='pca' is too heavy to dump, change to var_map=None"
+                )
                 out_dict["var_map"] = None
 
         out_dict["bounds"] = [m.bounds for m in self.maps]
@@ -201,12 +205,11 @@ class MapXbody:
             singlexb.set_bounds(bounds[0], bounds[1])
             singlexb.build_map_container()
             singlexb.mean.__coeffs__ = np.array(dictionary["maps"][0][m])
-            if new_mgp.var_map == 'simple':
+            if new_mgp.var_map == "simple":
                 singlexb.var.__coeffs__ = np.array(dictionary["maps"][1][m])
 
         return new_mgp
 
-
     def write(self, f, write_var):
         for m in self.maps:
             m.write(f, write_var)
@@ -398,10 +401,10 @@ class SingleMapXbody:
 
         if np.any(np.array(self.bounds[1]) <= 0.0):
             if force_block:
-                return np.zeros((n_grids, (e-s)*3))
+                return np.zeros((n_grids, (e - s) * 3))
             else:
-                return np.zeros((n_grids, e-s))
- 
+                return np.zeros((n_grids, e - s))
+
         grids = self.construct_grids()
         coords = np.zeros(
             (grids.shape[0], self.grid_dim * 3), dtype=np.float64
@@ -437,7 +440,12 @@ class SingleMapXbody:
                 fj_chunk = fj[gs:ge, :]
                 fdj_chunk = fdj[gs:ge, :]
                 kv_chunk = self.get_grid_kernel(
-                    kern_type, data, kernel_info, grid_chunk, fj_chunk, fdj_chunk,
+                    kern_type,
+                    data,
+                    kernel_info,
+                    grid_chunk,
+                    fj_chunk,
+                    fdj_chunk,
                 )
                 kern_vec.append(kv_chunk)
             kern_vec = np.hstack(kern_vec)
@@ -517,7 +525,7 @@ class SingleMapXbody:
             rebuild_container = True
 
             warnings.warn(
-                f"The minimal distance in training data is lower than "
+                "The minimal distance in training data is lower than "
                 f"the current lower bound, will reset lower bound to {lower_bound}"
             )
 
@@ -614,14 +622,13 @@ class SingleMapXbody:
                 f"The minimal distance {min_dist:.3f}"
                 f" is below the mgp lower bound {self.bounds[0]}",
             )
-        
+
         max_dist = np.max(lengths)
         if max_dist > np.min(self.bounds[1]):
             raise Exception(
                 self.species,
                 max_dist,
-                f"The atomic environment should have cutoff smaller"
-                f" than the GP cutoff"
+                f"The atomic environment should have cutoff smaller than the GP cutoff",
             )
 
         lengths = np.array(lengths)
@@ -669,9 +676,8 @@ class SingleMapXbody:
         vir *= self.bodies / 2
         return f, vir, v, e
 
-
     def write(self, f, write_var, permute=False):
-        """ 
+        """
         Write LAMMPS coefficient file
 
         This implementation only works for 2b and 3b. User should
@@ -703,7 +709,6 @@ class SingleMapXbody:
 
         self.write_flatten_coeff(f, coefs)
 
-
     def write_flatten_coeff(self, f, coefs):
         """
         flatten the coefficient and write it as
