@@ -17,6 +17,7 @@ from flare.output import Output
 from flare.utils.learner import is_std_in_bound
 from flare.utils.element_coder import NumpyEncoder
 
+
 class OTF:
     """Trains a Gaussian process force field on the fly during
         molecular dynamics.
@@ -93,25 +94,38 @@ class OTF:
     """
 
     def __init__(
-     self,
-     # md args
-     dt: float, number_of_steps: int, prev_pos_init: 'ndarray' = None,
-     rescale_steps: List[int] = [], rescale_temps: List[int] = [],
-     # flare args
-     gp: gp.GaussianProcess = None, calculate_energy: bool = False,
-     calculate_efs: bool = False, write_model: int = 0,
-     # otf args
-     std_tolerance_factor: float = 1, skip: int = 0,
-     init_atoms: List[int] = None, output_name: str = 'otf_run',
-     max_atoms_added: int = 1, freeze_hyps: int = 10,
-     # dft args
-     force_source: str = "qe", npool: int = None, mpi: str = "srun",
-     dft_loc: str = None, dft_input: str = None, dft_output='dft.out',
-     dft_kwargs=None,
-     store_dft_output: Tuple[Union[str, List[str]], str] = None,
-     # other args
-     n_cpus: int = 1, **kwargs):
-
+        self,
+        # md args
+        dt: float,
+        number_of_steps: int,
+        prev_pos_init: "ndarray" = None,
+        rescale_steps: List[int] = [],
+        rescale_temps: List[int] = [],
+        # flare args
+        gp: gp.GaussianProcess = None,
+        calculate_energy: bool = False,
+        calculate_efs: bool = False,
+        write_model: int = 0,
+        # otf args
+        std_tolerance_factor: float = 1,
+        skip: int = 0,
+        init_atoms: List[int] = None,
+        output_name: str = "otf_run",
+        max_atoms_added: int = 1,
+        freeze_hyps: int = 10,
+        # dft args
+        force_source: str = "qe",
+        npool: int = None,
+        mpi: str = "srun",
+        dft_loc: str = None,
+        dft_input: str = None,
+        dft_output="dft.out",
+        dft_kwargs=None,
+        store_dft_output: Tuple[Union[str, List[str]], str] = None,
+        # other args
+        n_cpus: int = 1,
+        **kwargs,
+    ):
 
         # set DFT
         self.dft_loc = dft_loc
@@ -127,7 +141,7 @@ class OTF:
         # set md
         self.dt = dt
         self.number_of_steps = number_of_steps
-        self.get_structure_from_input(prev_pos_init) # parse input file
+        self.get_structure_from_input(prev_pos_init)  # parse input file
         self.noa = self.structure.positions.shape[0]
         self.rescale_steps = rescale_steps
         self.rescale_temps = rescale_temps
@@ -145,12 +159,12 @@ class OTF:
         self.skip = skip
         self.max_atoms_added = max_atoms_added
         self.freeze_hyps = freeze_hyps
-        if init_atoms is None: # set atom list for initial dft run
+        if init_atoms is None:  # set atom list for initial dft run
             self.init_atoms = [int(n) for n in range(self.noa)]
         else:
             self.init_atoms = init_atoms
 
-        self.n_cpus = n_cpus # set number of cpus and npool for DFT runs
+        self.n_cpus = n_cpus  # set number of cpus and npool for DFT runs
         self.npool = npool
         self.mpi = mpi
 
@@ -163,19 +177,19 @@ class OTF:
 
         # Set the prediction function based on user inputs.
         # Force only prediction.
-        if (n_cpus > 1 and gp.per_atom_par and gp.parallel) and not \
-           (calculate_energy or calculate_efs):
+        if (n_cpus > 1 and gp.per_atom_par and gp.parallel) and not (
+            calculate_energy or calculate_efs
+        ):
             self.pred_func = predict.predict_on_structure_par
         elif not (calculate_energy or calculate_efs):
             self.pred_func = predict.predict_on_structure
         # Energy and force prediction.
-        elif (n_cpus > 1 and gp.per_atom_par and gp.parallel) and not \
-             (calculate_efs):
+        elif (n_cpus > 1 and gp.per_atom_par and gp.parallel) and not (calculate_efs):
             self.pred_func = predict.predict_on_structure_par_en
         elif not calculate_efs:
             self.pred_func = predict.predict_on_structure_en
         # Energy, force, and stress prediction.
-        elif (n_cpus > 1 and gp.per_atom_par and gp.parallel):
+        elif n_cpus > 1 and gp.per_atom_par and gp.parallel:
             self.pred_func = predict.predict_on_structure_efs_par
         else:
             self.pred_func = predict.predict_on_structure_efs
@@ -183,8 +197,8 @@ class OTF:
         # set logger
         self.output = Output(output_name, always_flush=True)
         self.output_name = output_name
-        self.gp_name = self.output_name + '_gp.json'
-        self.checkpt_name = self.output_name + '_checkpt.json'
+        self.gp_name = self.output_name + "_gp.json"
+        self.checkpt_name = self.output_name + "_checkpt.json"
 
         self.write_model = write_model
 
@@ -199,16 +213,24 @@ class OTF:
 
         optional_dict = {"Restart": self.curr_step}
         self.output.write_header(
-            str(self.gp), self.dt, self.number_of_steps, self.structure,
-            self.std_tolerance, optional_dict)
+            str(self.gp),
+            self.dt,
+            self.number_of_steps,
+            self.structure,
+            self.std_tolerance,
+            optional_dict,
+        )
 
         counter = 0
         self.start_time = time.time()
 
         while self.curr_step < self.number_of_steps:
             # run DFT and train initial model if first step and DFT is on
-            if (self.curr_step == 0) and (self.std_tolerance != 0) and \
-               (len(self.gp.training_data) == 0):
+            if (
+                (self.curr_step == 0)
+                and (self.std_tolerance != 0)
+                and (len(self.gp.training_data) == 0)
+            ):
 
                 # Are the recorded forces from the GP or DFT in ASE OTF?
                 # When DFT is called, ASE energy, forces, and stresses should
@@ -223,8 +245,11 @@ class OTF:
 
                 # get max uncertainty atoms
                 std_in_bound, target_atoms = is_std_in_bound(
-                    self.std_tolerance, self.gp.force_noise, self.structure,
-                    self.max_atoms_added)
+                    self.std_tolerance,
+                    self.gp.force_noise,
+                    self.structure,
+                    self.max_atoms_added,
+                )
 
                 if not std_in_bound:
                     # record GP forces
@@ -236,6 +261,8 @@ class OTF:
                     self.dft_step = True
                     self.run_dft()
                     dft_frcs = deepcopy(self.structure.forces)
+                    dft_stress = deepcopy(self.structure.stress)
+                    dft_energy = self.structure.potential_energy
 
                     # run MD step & record the state
                     self.record_state()
@@ -244,7 +271,12 @@ class OTF:
                     self.compute_mae(gp_frcs, dft_frcs)
 
                     # add max uncertainty atoms to training set
-                    self.update_gp(target_atoms, dft_frcs)
+                    self.update_gp(
+                        target_atoms,
+                        dft_frcs,
+                        dft_stress=dft_stress,
+                        dft_energy=dft_energy,
+                    )
 
             # write gp forces
             if counter >= self.skip and not self.dft_step:
@@ -254,9 +286,12 @@ class OTF:
 
             counter += 1
             # TODO: Reinstate velocity rescaling.
-            self.md_step()
+            self.md_step() # update positions by Verlet
+            self.rescale_temperature(self.structure.positions)
             self.curr_step += 1
-            self.checkpoint()
+
+            if self.write_model == 3:
+                self.checkpoint()
 
         self.output.conclude_run()
 
@@ -265,35 +300,45 @@ class OTF:
             self.checkpoint()
 
     def get_structure_from_input(self, prev_pos_init):
-        positions, species, cell, masses = \
-            self.dft_module.parse_dft_input(self.dft_input)
+        positions, species, cell, masses = self.dft_module.parse_dft_input(
+            self.dft_input
+        )
 
         self.structure = struc.Structure(
-            cell=cell, species=species, positions=positions, mass_dict=masses,
-            prev_positions=prev_pos_init, species_labels=species)
+            cell=cell,
+            species=species,
+            positions=positions,
+            mass_dict=masses,
+            prev_positions=prev_pos_init,
+            species_labels=species,
+        )
 
     def initialize_train(self):
         # call dft and update positions
         self.run_dft()
         dft_frcs = deepcopy(self.structure.forces)
+        dft_stress = deepcopy(self.structure.stress)
+        dft_energy = self.structure.potential_energy
 
         self.update_temperature()
         self.record_state()
 
         # make initial gp model and predict forces
-        self.update_gp(self.init_atoms, dft_frcs)
+        self.update_gp(
+            self.init_atoms, dft_frcs, dft_stress=dft_stress, dft_energy=dft_energy
+        )
 
     def compute_properties(self):
-        '''
+        """
         In ASE-OTF, it will be replaced by subclass method
-        '''
+        """
         self.gp.check_L_alpha()
         self.pred_func(self.structure, self.gp, self.n_cpus)
 
     def md_step(self):
-        '''
+        """
         Take an MD step. This updates the positions of the structure.
-        '''
+        """
         md.update_positions(self.dt, self.noa, self.structure)
 
     def write_gp(self):
@@ -308,18 +353,22 @@ class OTF:
 
         Calculates DFT forces on atoms in the current structure."""
 
-        f = logging.getLogger(self.output.basename+'log')
-        f.info('\nCalling DFT...\n')
+        f = logging.getLogger(self.output.basename + "log")
+        f.info("\nCalling DFT...\n")
 
         # calculate DFT forces
         # TODO: Return stress and energy
         forces = self.dft_module.run_dft_par(
-            self.dft_input, self.structure, self.dft_loc, n_cpus=self.n_cpus,
-            dft_out=self.dft_output, npool=self.npool, mpi=self.mpi,
-            dft_kwargs=self.dft_kwargs)
+            self.dft_input,
+            self.structure,
+            self.dft_loc,
+            n_cpus=self.n_cpus,
+            dft_out=self.dft_output,
+            npool=self.npool,
+            mpi=self.mpi,
+            dft_kwargs=self.dft_kwargs,
+        )
 
-        # Note: also need to update stresses when performing a simulation
-        # in the NPT ensemble.
         self.structure.forces = forces
 
         # write wall time of DFT calculation
@@ -338,10 +387,15 @@ class OTF:
             else:
                 to_copy = target_files
             for ofile in to_copy:
-                copyfile(ofile, dest+'/'+dt_string+ofile)
+                copyfile(ofile, dest + "/" + dt_string + ofile)
 
-    def update_gp(self, train_atoms: List[int], dft_frcs: 'ndarray',
-                  dft_energy: float = None, dft_stress: 'ndarray' = None):
+    def update_gp(
+        self,
+        train_atoms: List[int],
+        dft_frcs: "ndarray",
+        dft_energy: float = None,
+        dft_stress: "ndarray" = None,
+    ):
         """
         Updates the current GP model.
 
@@ -354,13 +408,18 @@ class OTF:
         self.output.add_atom_info(train_atoms, self.structure.stds)
 
         # update gp model
-        self.gp.update_db(self.structure, dft_frcs, custom_range=train_atoms,
-                          energy=dft_energy, stress=dft_stress)
+        self.gp.update_db(
+            self.structure,
+            dft_frcs,
+            custom_range=train_atoms,
+            energy=dft_energy,
+            stress=dft_stress,
+        )
 
         self.gp.set_L_alpha()
 
         # write model
-        if (self.dft_count-1) < self.freeze_hyps:
+        if (self.dft_count - 1) < self.freeze_hyps:
             self.train_gp()
             if self.write_model == 2:
                 self.write_gp()
@@ -370,27 +429,31 @@ class OTF:
     def train_gp(self):
         """Optimizes the hyperparameters of the current GP model."""
 
-        self.gp.train(logger_name=self.output.basename+'hyps')
+        self.gp.train(logger_name=self.output.basename + "hyps")
 
         hyps, labels = self.gp.hyps_and_labels
         if labels is None:
             labels = self.gp.hyp_labels
 
-        self.output.write_hyps(labels, hyps,
-                               self.start_time,
-                               self.gp.likelihood, self.gp.likelihood_gradient,
-                               hyps_mask=self.gp.hyps_mask)
+        self.output.write_hyps(
+            labels,
+            hyps,
+            self.start_time,
+            self.gp.likelihood,
+            self.gp.likelihood_gradient,
+            hyps_mask=self.gp.hyps_mask,
+        )
 
     def compute_mae(self, gp_frcs, dft_frcs):
         mae = np.mean(np.abs(gp_frcs - dft_frcs))
         mac = np.mean(np.abs(dft_frcs))
 
-        f = logging.getLogger(self.output.basename+'log')
-        f.info(f'mean absolute error: {mae:.4f} eV/A')
-        f.info(f'mean absolute dft component: {mac:.4f} eV/A')
+        f = logging.getLogger(self.output.basename + "log")
+        f.info(f"mean absolute error: {mae:.4f} eV/A")
+        f.info(f"mean absolute dft component: {mac:.4f} eV/A")
 
-    def update_positions(self, new_pos: 'ndarray'):
-        """Performs a Verlet update of the atomic positions.
+    def rescale_temperature(self, new_pos: "ndarray"):
+        """Change the previous positions to update the temperature
 
         Args:
             new_pos (np.ndarray): Positions of atoms in the next MD frame.
@@ -399,12 +462,9 @@ class OTF:
             rescale_ind = self.rescale_steps.index(self.curr_step)
             temp_fac = self.rescale_temps[rescale_ind] / self.temperature
             vel_fac = np.sqrt(temp_fac)
-            self.structure.prev_positions = \
+            self.structure.prev_positions = (
                 new_pos - self.velocities * self.dt * vel_fac
-        else:
-            self.structure.prev_positions = self.structure.positions
-        self.structure.positions = new_pos
-        self.structure.positions[:] = self.structure.wrap_positions()
+            )
 
     def update_temperature(self):
         """Updates the instantaneous temperatures of the system.
@@ -412,63 +472,71 @@ class OTF:
         Args:
             new_pos (np.ndarray): Positions of atoms in the next MD frame.
         """
-        KE, temperature, velocities = \
-            md.calculate_temperature(self.structure, self.dt, self.noa)
+        KE, temperature, velocities = md.calculate_temperature(
+            self.structure, self.dt, self.noa
+        )
         self.KE = KE
         self.temperature = temperature
         self.velocities = velocities
 
     def record_state(self):
         self.output.write_md_config(
-            self.dt, self.curr_step, self.structure, self.temperature,
-            self.KE, self.start_time, self.dft_step, self.velocities)
+            self.dt,
+            self.curr_step,
+            self.structure,
+            self.temperature,
+            self.KE,
+            self.start_time,
+            self.dft_step,
+            self.velocities,
+        )
 
     def as_dict(self):
         self.dft_module = self.dft_module.__name__
         out_dict = deepcopy(dict(vars(self)))
         self.dft_module = eval(self.dft_module)
 
-        out_dict['gp'] = self.gp_name
-        out_dict['structure'] = self.structure.as_dict()
+        out_dict["gp"] = self.gp_name
+        out_dict["structure"] = self.structure.as_dict()
 
-        for key in ['output', 'pred_func']:
+        for key in ["output", "pred_func"]:
             out_dict.pop(key)
 
         return out_dict
 
     @staticmethod
     def from_dict(in_dict):
-        if in_dict['write_model'] <= 1: # TODO: detect GP version
+        if in_dict["write_model"] <= 1:  # TODO: detect GP version
             warnings.warn("The GP model might not be the latest")
-       
-        gp_model = gp.GaussianProcess.from_file(in_dict['gp'])
-        in_dict['gp'] = gp_model
-        in_dict['structure'] = struc.Structure.from_dict(in_dict['structure'])
 
-        if 'flare.dft_interface' in in_dict['dft_module']:
-            for dft_name in ['qe', 'cp2k', 'vasp']:
-                if dft_name in in_dict['dft_module']:
-                    in_dict['force_source'] = dft_name
+        gp_model = gp.GaussianProcess.from_file(in_dict["gp"])
+        in_dict["gp"] = gp_model
+        in_dict["structure"] = struc.Structure.from_dict(in_dict["structure"])
+
+        if "flare.dft_interface" in in_dict["dft_module"]:
+            for dft_name in ["qe", "cp2k", "vasp"]:
+                if dft_name in in_dict["dft_module"]:
+                    in_dict["force_source"] = dft_name
                     break
-        else: # if force source is a module
-            in_dict['force_source'] = eval(in_dict['dft_module'])
+        else:  # if force source is a module
+            in_dict["force_source"] = eval(in_dict["dft_module"])
 
         new_otf = OTF(**in_dict)
-        new_otf.structure = in_dict['structure']
-        new_otf.dft_count = in_dict['dft_count']
-        new_otf.curr_step = in_dict['curr_step']
+        new_otf.structure = in_dict["structure"]
+        new_otf.dft_count = in_dict["dft_count"]
+        new_otf.curr_step = in_dict["curr_step"]
         return new_otf
 
     def checkpoint(self):
         name = self.checkpt_name
-        if '.json' != name[-5:]:
-            name += '.json'
-        with open(name, 'w') as f:
+        if ".json" != name[-5:]:
+            name += ".json"
+        with open(name, "w") as f:
             json.dump(self.as_dict(), f, cls=NumpyEncoder)
 
     @classmethod
     def from_checkpoint(cls, filename):
-        with open(filename, 'r') as f:
+        with open(filename, "r") as f:
             otf_model = cls.from_dict(json.loads(f.readline()))
 
         return otf_model
