@@ -39,10 +39,13 @@ class Output:
     :type always_flus: bool, optional
     """
 
-    def __init__(self, basename: str = 'otf_run',
-                 verbose: str = 'INFO',
-                 print_as_xyz: bool = False,
-                 always_flush: bool = False):
+    def __init__(
+        self,
+        basename: str = "otf_run",
+        verbose: str = "INFO",
+        print_as_xyz: bool = False,
+        always_flush: bool = False,
+    ):
         """
         Construction. Open files.
         """
@@ -50,9 +53,9 @@ class Output:
         self.print_as_xyz = print_as_xyz
         self.always_flush = always_flush
 
-        filesuffix = {'log': '.out', 'hyps': '-hyps.dat'}
+        filesuffix = {"log": ".out", "hyps": "-hyps.dat"}
         if print_as_xyz:
-            filesuffix['xyz'] = '.xyz'
+            filesuffix["xyz"] = ".xyz"
 
         self.logger = []
         for filetype in filesuffix:
@@ -63,16 +66,16 @@ class Output:
         destruction function that closes all files
         """
 
-        logger = logging.getLogger(self.basename+'log')
-        logger.info('-' * 20)
-        logger.info('Run complete.')
+        logger = logging.getLogger(self.basename + "log")
+        logger.info("-" * 20)
+        logger.info("Run complete.")
         if extra_strings is not None:
             for string in extra_strings:
                 logger.info(string)
         logging.shutdown()
         self.logger = []
 
-    def open_new_log(self, filetype: str, suffix: str, verbose='info'):
+    def open_new_log(self, filetype: str, suffix: str, verbose="info"):
         """
         Open files.  If files with the same
         name are exist, they are backed up with a suffix "-bak".
@@ -83,13 +86,15 @@ class Output:
         """
 
         if filetype not in self.logger:
-            set_logger(self.basename+filetype, stream=False,
-                       fileout_name=self.basename+suffix,
-                       verbose=verbose)
+            set_logger(
+                self.basename + filetype,
+                stream=False,
+                fileout_name=self.basename + suffix,
+                verbose=verbose,
+            )
             self.logger += [filetype]
 
-    def write_to_log(self, logstring: str, name: str = "log",
-                     flush: bool = False):
+    def write_to_log(self, logstring: str, name: str = "log", flush: bool = False):
         """
         Write any string to logfile
 
@@ -97,17 +102,21 @@ class Output:
         :param name: the key name of the file to logger named 'log'
         :param flush: whether it should be flushed
         """
-        logger = logging.getLogger(self.basename+name)
+        logger = logging.getLogger(self.basename + name)
         logger.info(logstring)
 
         if flush or self.always_flush:
             logger.handlers[0].flush()
 
-    def write_header(self, gp_str: str,
-                     dt: float = None,
-                     Nsteps: int = None, structure: Structure = None,
-                     std_tolerance: Union[float, int] = None,
-                     optional: dict = None):
+    def write_header(
+        self,
+        gp_str: str,
+        dt: float = None,
+        Nsteps: int = None,
+        structure: Structure = None,
+        std_tolerance: Union[float, int] = None,
+        optional: dict = None,
+    ):
         """
         TO DO: this should be replace by the string method of GP and OTF, GPFA
 
@@ -122,37 +131,37 @@ class Output:
         :param optional: a dictionary of all the other parameters
         """
 
-        f = logging.getLogger(self.basename+'log')
-        f.info(f'{datetime.datetime.now()}')
+        f = logging.getLogger(self.basename + "log")
+        f.info(f"{datetime.datetime.now()}")
 
         if isinstance(std_tolerance, tuple):
-            std_string = 'Relative uncertainty tolerance: ' \
-                         f'{std_tolerance[0]} times noise hyperparameter \n'
-            std_string += 'Absolute uncertainty tolerance: ' \
-                          f'{std_tolerance[1]} eV/A\n'
+            std_string = (
+                "Relative uncertainty tolerance: "
+                f"{std_tolerance[0]} times noise hyperparameter \n"
+            )
+            std_string += f"Absolute uncertainty tolerance: {std_tolerance[1]} eV/A\n"
         elif std_tolerance < 0:
-            std_string = \
-                f'Uncertainty tolerance: {np.abs(std_tolerance)} eV/A\n'
+            std_string = f"Uncertainty tolerance: {np.abs(std_tolerance)} eV/A\n"
         elif std_tolerance > 0:
-            std_string = \
-                f'Uncertainty tolerance: {np.abs(std_tolerance)} ' \
-                'times noise hyperparameter \n'
+            std_string = (
+                f"Uncertainty tolerance: {np.abs(std_tolerance)} "
+                "times noise hyperparameter \n"
+            )
         else:
-            std_string = ''
+            std_string = ""
 
-        headerstring = '\n'
+        headerstring = "\n"
         headerstring += gp_str
-        headerstring += '\n'
+        headerstring += "\n"
         headerstring += std_string
         if dt is not None:
-            headerstring += f'Timestep (ps): {dt}\n'
-        headerstring += f'Number of frames: {Nsteps}\n'
+            headerstring += f"Timestep (ps): {dt}\n"
+        headerstring += f"Number of frames: {Nsteps}\n"
         if structure is not None:
-            headerstring += f'Number of atoms: {structure.nat}\n'
-            headerstring += \
-                f'System species: {set(structure.species_labels)}\n'
-            headerstring += 'Periodic cell (A): \n'
-            headerstring += str(np.array(structure.cell))+'\n'
+            headerstring += f"Number of atoms: {structure.nat}\n"
+            headerstring += f"System species: {set(structure.species_labels)}\n"
+            headerstring += "Periodic cell (A): \n"
+            headerstring += str(np.array(structure.cell)) + "\n"
 
         if optional:
             for key, value in optional.items():
@@ -160,37 +169,43 @@ class Output:
 
         # report previous positions
         if structure is not None:
-            headerstring += '\nPrevious positions (A):\n'
+            headerstring += "\nPrevious positions (A):\n"
             for i in range(len(structure.positions)):
-                headerstring += f'{structure.species_labels[i]:5}'
+                headerstring += f"{structure.species_labels[i]:5}"
                 for j in range(3):
-                    headerstring += f'{structure.prev_positions[i][j]:10.4f}'
-                headerstring += '\n'
-        headerstring += '-' * 80
+                    headerstring += f"{structure.prev_positions[i][j]:10.4f}"
+                headerstring += "\n"
+        headerstring += "-" * 80
 
         f.info(headerstring)
 
         if self.always_flush:
             f.handlers[0].flush()
 
-
     def write_md_header(self, dt, curr_step, dft_step):
-        string = ''
+        string = ""
         # Mark if a frame had DFT forces with an asterisk
         if not dft_step:
-            string += '-' * 80 + '\n'
+            string += "-" * 80 + "\n"
             string += f"-Frame: {curr_step} "
         else:
             string += f"\n*-Frame: {curr_step} "
 
-        string += f'\nSimulation Time: {(dt * curr_step):.3} ps \n'
+        string += f"\nSimulation Time: {(dt * curr_step):10.3f} ps \n"
         return string
 
-
     def write_md_config(
-            self, dt, curr_step, structure, temperature, KE, start_time, dft_step,
-            velocities):
-        """ write md configuration in log file
+        self,
+        dt,
+        curr_step,
+        structure,
+        temperature,
+        KE,
+        start_time,
+        dft_step,
+        velocities,
+    ):
+        """write md configuration in log file
 
         :param dt: timestemp of OTF MD
         :param curr_step: current timestep of OTF MD
@@ -209,99 +224,114 @@ class Output:
 
         # Construct Header line
         n_space = 30
-        string += str.ljust('El', 5)
-        string += str.center('Position (A)', n_space)
-        string += ' ' * 4
+        string += str.ljust("El", 5)
+        string += str.center("Position (A)", n_space)
+        string += " " * 4
         if not dft_step:
-            string += str.center('GP Force (ev/A)', n_space)
-            string += ' ' * 4
+            string += str.center("GP Force (ev/A)", n_space)
+            string += " " * 4
         else:
-            string += str.center('DFT Force (ev/A)', n_space)
-            string += ' ' * 4
-        string += str.center('Std. Dev (ev/A)', n_space) + ' ' * 4
-        string += str.center('Velocities (A/ps)', n_space) + '\n'
+            string += str.center("DFT Force (ev/A)", n_space)
+            string += " " * 4
+        string += str.center("Std. Dev (ev/A)", n_space) + " " * 4
+        string += str.center("Velocities (A/ps)", n_space) + "\n"
 
         # Construct atom-by-atom description
         for i in range(len(structure.positions)):
-            string += f'{structure.species_labels[i]:5}'
+            string += f"{structure.species_labels[i]:5}"
             # string += '\t'
             for j in range(3):
-                string += f'{structure.positions[i][j]:10.4f}'
-            string += ' ' * 4
+                string += f"{structure.positions[i][j]:10.4f}"
+            string += " " * 4
             for j in range(3):
-                string += f'{structure.forces[i][j]:10.4f}'
-            string += ' ' * 4
+                string += f"{structure.forces[i][j]:10.4f}"
+            string += " " * 4
             for j in range(3):
-                string += f'{structure.stds[i][j]:10.4f}'
-            string += ' ' * 4
+                string += f"{structure.stds[i][j]:10.4f}"
+            string += " " * 4
             for j in range(3):
-                string += f'{velocities[i][j]:10.4f}'
-            string += '\n'
+                string += f"{velocities[i][j]:10.4f}"
+            string += "\n"
 
-        string += '\n'
+        string += "\n"
 
         # Report cell if stress attribute is present.
         if structure.stress is not None:
-            string += 'Periodic cell (A): \n'
-            string += str(np.array(structure.cell))+'\n\n'
+            string += "Periodic cell (A): \n"
+            string += str(np.array(structure.cell)) + "\n\n"
 
         # Report stress tensor.
         pressure = None
         if structure.stress is not None:
             stress_tensor = structure.stress * eva_to_gpa  # Convert to GPa
-            s8 = ' ' * 8
-            string += 'Stress tensor (GPa):\n'
-            string += ' ' * 7 + 'xx' + s8 + 'yy' + s8 + 'zz' + s8 + 'yz' + \
-                s8 + 'xz' + s8 + 'xy\n'
+            s8 = " " * 8
+            string += "Stress tensor (GPa):\n"
+            string += (
+                " " * 7
+                + "xx"
+                + s8
+                + "yy"
+                + s8
+                + "zz"
+                + s8
+                + "yz"
+                + s8
+                + "xz"
+                + s8
+                + "xy\n"
+            )
             for p in range(6):
-                string += f'{stress_tensor[p]:10.3f}'
-            string += '\n\n'
-            pressure = \
-                (stress_tensor[0] + stress_tensor[1] + stress_tensor[2]) / 3
+                string += f"{stress_tensor[p]:10.3f}"
+            string += "\n\n"
+            pressure = (stress_tensor[0] + stress_tensor[1] + stress_tensor[2]) / 3
 
         # Report stress tensor uncertainties.
         if structure.stress_stds is not None:
             stress_stds = structure.stress_stds * eva_to_gpa  # Convert to GPa
-            string += 'Stress tensor uncertainties (GPa):\n'
+            string += "Stress tensor uncertainties (GPa):\n"
             for p in range(6):
-                string += f'{stress_stds[p]:10.3f}'
-            string += '\n\n'
+                string += f"{stress_stds[p]:10.3f}"
+            string += "\n\n"
 
         # Report pressure.
         if pressure is not None:
-            string += f'Pressure (GPa): {pressure:.6f} \n'
+            string += f"Pressure (GPa): {pressure:.6f} \n"
 
-        string += f'Temperature (K): {temperature:.2f} \n'
-        string += f'Kinetic energy (eV): {KE:.6f} \n'
+        string += f"Temperature (K): {temperature:.2f} \n"
+        string += f"Kinetic energy (eV): {KE:.6f} \n"
 
         # Report potential energy.
         if structure.potential_energy is not None:
-            string += \
-                f'Potential energy (eV): {structure.potential_energy:.6f} \n'
+            string += f"Potential energy (eV): {structure.potential_energy:.6f} \n"
 
         # Report potential energy uncertainty.
         if structure.local_energy_stds is not None:
-            pot_en_std = np.sqrt(np.sum(structure.local_energy_stds**2))
-            string += f'Uncertainty (eV): {pot_en_std:.6f} \n'
+            pot_en_std = np.sqrt(np.sum(structure.local_energy_stds ** 2))
+            string += f"Uncertainty (eV): {pot_en_std:.6f} \n"
 
         # Report total energy.
         if structure.potential_energy is not None:
             tot_en = KE + structure.potential_energy
-            string += f'Total energy (eV): {tot_en:.6f} \n'
+            string += f"Total energy (eV): {tot_en:.6f} \n"
 
-        logger = logging.getLogger(self.basename+'log')
+        logger = logging.getLogger(self.basename + "log")
         logger.info(string)
         self.write_wall_time(start_time)
 
         if self.always_flush:
             logger.handlers[0].flush()
 
-
-    def write_xyz_config(self, curr_step, structure,
-                         forces: np.array = None, stds: np.array = None,
-                         dft_forces: np.array = None, dft_energy=0,
-                         predict_energy=float("nan")):
-        """ write atomic configuration in xyz file
+    def write_xyz_config(
+        self,
+        curr_step,
+        structure,
+        forces: np.array = None,
+        stds: np.array = None,
+        dft_forces: np.array = None,
+        dft_energy=0,
+        predict_energy=float("nan"),
+    ):
+        """write atomic configuration in xyz file
 
         :param curr_step: Int, number of frames to note in the comment line
         :param structure: Structure, contain positions and forces
@@ -312,22 +342,29 @@ class Output:
         :return:
         """
 
-        xyz_str = structure.to_xyz(extended_xyz=True,
-               print_stds=True, print_forces=True,
-               print_max_stds=False, print_energies=True,
-               predict_energy=predict_energy,
-               dft_forces=dft_forces, dft_energy=dft_energy,
-               timestep=curr_step,
-               write_file='', append=False)
+        xyz_str = structure.to_xyz(
+            extended_xyz=True,
+            print_stds=True,
+            print_forces=True,
+            print_max_stds=False,
+            print_energies=True,
+            predict_energy=predict_energy,
+            dft_forces=dft_forces,
+            dft_energy=dft_energy,
+            timestep=curr_step,
+            write_file="",
+            append=False,
+        )
 
-        logger = logging.getLogger(self.basename+'xyz')
+        logger = logging.getLogger(self.basename + "xyz")
         logger.info(xyz_str)
         if self.always_flush:
             logger.handlers[0].flush()
 
-    def write_hyps(self, hyp_labels, hyps, start_time, like, like_grad,
-                   name='log', hyps_mask=None):
-        """ write hyperparameters to logfile
+    def write_hyps(
+        self, hyp_labels, hyps, start_time, like, like_grad, name="log", hyps_mask=None
+    ):
+        """write hyperparameters to logfile
 
         :param name:
         :param hyp_labels: labels for hyper-parameters. can be None
@@ -338,19 +375,21 @@ class Output:
 
         :return:
         """
-        f = logging.getLogger(self.basename+name)
+        f = logging.getLogger(self.basename + name)
 
-        f.info('\nGP hyperparameters: ')
+        f.info("\nGP hyperparameters: ")
 
         if hyp_labels is not None:
             for i, label in enumerate(hyp_labels):
-                f.info(f'Hyp{i} : {label:30s} = {hyps[i]:.4f}')
+                f.info(f"Hyp{i} : {label:30s} = {hyps[i]:.4f}")
         else:
             for i, hyp in enumerate(hyps):
-                f.info(f'Hyp{i} : {hyp:.4f}')
-
-        f.info(f'Likelihood: {like:.4f}')
-        f.info(f'Likelihood gradient: {like_grad}')
+                f.info(f"Hyp{i} : {hyp:.4f}")
+        if isinstance(like,list):
+            f.info(f"Likelihood: {np.round(like,4)}")
+        else:
+            f.info(f"Likelihood: {like:.4f}")
+        f.info(f"Likelihood gradient: {like_grad}")
 
         if start_time:
             self.write_wall_time(start_time)
@@ -360,23 +399,34 @@ class Output:
 
     def write_wall_time(self, start_time):
         time_curr = time.time() - start_time
-        f = logging.getLogger(self.basename+'log')
-        f.info(f'Wall time from start: {time_curr:.2f} s')
+        f = logging.getLogger(self.basename + "log")
+        f.info(f"Wall time from start: {time_curr:.2f} s")
 
     def conclude_dft(self, dft_count, start_time):
-        f = logging.getLogger(self.basename+'log')
-        f.info('DFT run complete.')
-        f.info(f'Number of DFT calls: {dft_count}')
+        f = logging.getLogger(self.basename + "log")
+        f.info("DFT run complete.")
+        f.info(f"Number of DFT calls: {dft_count}")
         self.write_wall_time(start_time)
 
     def add_atom_info(self, train_atoms, stds):
-        f = logging.getLogger(self.basename+'log')
-        f.info(f'Adding atom {train_atoms} to the training set.')
-        f.info(f'Uncertainty: {stds[train_atoms[0]]}')
+        f = logging.getLogger(self.basename + "log")
+        f.info(f"Adding atom {train_atoms} to the training set.")
+        f.info(f"Uncertainty: {stds[train_atoms[0]]}")
 
     def write_gp_dft_comparison(
-            self, curr_step, frame, start_time, dft_forces, dft_energy, error,
-            local_energies=None, KE=None, mgp=False):
+        self,
+        curr_step,
+        frame,
+        start_time,
+        dft_forces,
+        dft_energy,
+        error,
+        local_energies=None,
+        KE=None,
+        mgp=False,
+        cell=None,
+        stress=None,
+    ):
         """Write the comparison to logfile.
 
         :param curr_step: current timestep
@@ -388,46 +438,56 @@ class Output:
         :param error: list of force differences between DFT and GP prediction
         :param local_energies: local atomic energy
         :param KE: total kinetic energy
+        :param cell: print the unit cell of the structure
+        :param stress: print the stress acting on the cell
 
         :return:
         """
 
-        string = ''
+        string = ""
 
         # Mark if a frame had DFT forces with an asterisk
         string += f"\n*-Frame: {curr_step}"
 
         # Construct Header line
-        string += '\nEl  Position (A) \t\t\t\t '
+        string += "\nEl  Position (A) \t\t\t\t "
         if mgp:
-            string += 'M'
-        string += 'GP Force (ev/A)  \t\t\t\t'
-        string += 'Std. Dev (ev/A) \t\t\t\t'
-        string += 'DFT Force (ev/A)  \t\t\t\t \n'
+            string += "M"
+        string += "GP Force (ev/A)  \t\t\t\t"
+        string += "Std. Dev (ev/A) \t\t\t\t"
+        string += "DFT Force (ev/A)  \t\t\t\t \n"
 
         # Construct atom-by-atom description
         for i in range(len(frame.positions)):
             string += f"{frame.species_labels[i]} "
             for j in range(3):
                 string += f"{frame.positions[i][j]:10.5} "
-            string += '\t'
+            string += "\t"
             for j in range(3):
                 string += f"{frame.forces[i][j]:10.5} "
-            string += '\t'
+            string += "\t"
             for j in range(3):
                 string += f"{frame.stds[i][j]:10.5} "
-            string += '\t'
+            string += "\t"
             for j in range(3):
                 string += f"{dft_forces[i][j]:10.5} "
-            string += '\n'
+            string += "\n"
 
-        string += '\n'
+        string += "\n"
 
+        # Print stress & cell related parameters
+        if cell is not None:
+            rounded_cell = np.round(cell, 4)
+            string += f"cell: {[list(vec) for vec in rounded_cell]} \n"
+        if stress:
+            raise NotImplementedError
+
+        # Compute errors and errors by species
         mae = np.nanmean(error) * 1000
         mac = np.mean(np.abs(dft_forces)) * 1000
-        string += f'mean absolute error: {mae:.2f} meV/A\n'
-        string += f'mean absolute dft component: {mac:.2f} meV/A\n'
-        stat = f'{curr_step} {mae:.2} {mac:.2}'
+        string += f"mean absolute error: {mae:.2f} meV/A\n"
+        string += f"mean absolute dft component: {mac:.2f} meV/A\n"
+        stat = f"{curr_step} {mae:.2} {mac:.2}"
 
         mae_per_species = {}
         count_per_species = {}
@@ -447,29 +507,34 @@ class Output:
         string += "mae per species\n"
         for ele in species:
             if count_per_species[ele] > 0:
-                mae_per_species[ele] /= (count_per_species[ele] * 3)
+                mae_per_species[ele] /= count_per_species[ele] * 3
                 mae_per_species[ele] *= 1000  # Put in meV/A
                 string += f"type {ele} mae: {mae_per_species[ele]:.2f} meV/A\n"
-            stat += f' {mae_per_species[ele]:.2f}'
+            stat += f" {mae_per_species[ele]:.2f}"
 
         # calculate potential and total energy
         if local_energies is not None:
             pot_en = 0
             pot_en = np.sum(local_energies)
             tot_en = KE + pot_en
-            string += f'potential energy: {pot_en:10.6} eV (DFT: {dft_energy} eV\n'
-            string += f'total energy: {tot_en:10.6} eV \n'
-            stat += f' {pot_en:10.6} {tot_en:10.6}'
+            string += f"potential energy: {pot_en:10.6} eV (DFT: {dft_energy} eV\n"
+            string += f"total energy: {tot_en:10.6} eV \n"
+            stat += f" {pot_en:10.6} {tot_en:10.6}"
         else:
             pot_en = float("nan")
 
         if self.print_as_xyz:
-            self.write_xyz_config(curr_step, frame, forces=frame.forces,
-                                  stds=frame.stds, dft_forces=dft_forces,
-                                  dft_energy=dft_energy,
-                                  predict_energy=pot_en)
+            self.write_xyz_config(
+                curr_step,
+                frame,
+                forces=frame.forces,
+                stds=frame.stds,
+                dft_forces=dft_forces,
+                dft_energy=dft_energy,
+                predict_energy=pot_en,
+            )
 
-        f = logging.getLogger(self.basename+'log')
+        f = logging.getLogger(self.basename + "log")
         f.info(string)
         self.write_wall_time(start_time)
 
@@ -481,13 +546,13 @@ class Output:
 
 
 def add_stream(logger: Logger, verbose: str = "info"):
-    '''
+    """
     set up screen sctream handler to the logger with handlers
 
     :param logger: the logger
     :param verbose: verbose level
     :type verbose: str
-    '''
+    """
 
     stream_defined = False
     for handler in logger.handlers:
@@ -503,7 +568,7 @@ def add_stream(logger: Logger, verbose: str = "info"):
 
 
 def add_file(logger: Logger, filename: str, verbose: str = "info"):
-    '''
+    """
     set up file handler to the logger with handlers
 
     :param logger: the logger
@@ -511,7 +576,7 @@ def add_file(logger: Logger, filename: str, verbose: str = "info"):
     :type filename: str
     :param verbose: verbose level
     :type verbose: str
-    '''
+    """
 
     file_defined = False
     for handler in logger.handlers:
@@ -522,7 +587,7 @@ def add_file(logger: Logger, filename: str, verbose: str = "info"):
 
         # back up
         if isfile(filename):
-            movefile(filename, filename+"-bak")
+            movefile(filename, filename + "-bak")
 
         fh = FileHandler(filename)
         verbose = getattr(logging, verbose.upper())
@@ -531,9 +596,10 @@ def add_file(logger: Logger, filename: str, verbose: str = "info"):
         logger.addHandler(fh)
 
 
-def set_logger(name: str, stream: bool, fileout_name: str = None,
-               verbose: str = "info"):
-    '''
+def set_logger(
+    name: str, stream: bool, fileout_name: str = None, verbose: str = "info"
+):
+    """
     set up a logger with handlers
 
     :param name: unique name of the logger in logging module
@@ -544,7 +610,7 @@ def set_logger(name: str, stream: bool, fileout_name: str = None,
     :type fileout_name: str
     :param verbose: verbose level
     :type verbose: str
-    '''
+    """
     logger = logging.getLogger(name)
     logger.propagate = False
     logger.handlers = []
